function PART=dat_part(input,d)
% DPART  PART=dat_part(ADJ,DATA)
%           Partitions a system described as an adjacency binary 
%           matrix (ADJ) acording to data availability (DATA).
%           PART is a structure that for each data series available 
%           contains the subcomponents [y,x,beta,eqs] corresponding 
%           to the dependent variable, independent variables, 
%           parameters that can be estimated and equations involved
%           in the estimation. If the names of the variables (NMS) are
%           provided, the rutine will generate a readable output and report
%			the parameters that are not reachible form the available data.

% The algorithm is based on a breath-first search taking each data
% series available as the starting node and clipping the tree 
% whenever another data series or a parameter is found.

%
%                                                Rogelio Oliva
%                                                July 27, 1998.  
%                                                Boston, MA
%================================================================

if isa(input,'struct') 
	s=input.adj; 
else
	s=input;
end;

fprintf('Identifying model parameters ...\n') 
np=find(sum(s)==0);                         % elements with no predecesors
p=(setdiff(np,d));                          % eliminate data series


for i=1:size(d,2)                           % for every data series available
    PART(i).y=d(i);                         % set it as independent variable
    PART(i).x=[];
    PART(i).beta=[];
    PART(i).eqs=d(i);                       % include as first equation

    color=zeros(1,size(s,1));               % set color vector to 0 (white)
    Q=d(i);                                 % initialize queue
    k=1;                                    % initialize pointer to queue
    while k<=size(Q,2)                      % while elements in queue
        u=Q(k);
        for j=find(s(:,u))'                 % for each predecesor
            if color(j)==0                  % if not discovered
                color(j)=1;                 % mark j as discovered (gray)
                if ismember(j,d)            % if data series
                    PART(i).x(size(PART(i).x,2)+1)=j;           % add j to x
                else
                    if ismember(j,p)        % if parameter
                        PART(i).beta(size(PART(i).beta,2)+1)=j; % add j to beta
                    else                    % if not in d or p
                        PART(i).eqs(size(PART(i).eqs,2)+1)=j;   % add j to eqs
                        Q(size(Q,2)+1)=j;   % add j to the end of Queue
                    end
                end
            end
        end
        k=k+1;                              % remove u from Queue
        color(u)=2;                         % mark u as explored (blak)
    end
end

nc=setdiff(p,unique([PART.beta]));

if isa(input,'struct') 
    for i=1:size(PART,2)
        fprintf('y:    \n')
        fprintf('      ')
        fprintf(input.nms(PART(i).y,:))
        fprintf('\n')
        fprintf('x:    \n')
        for j=1:size(PART(i).x,2)
            fprintf('      ')
            fprintf(input.nms(PART(i).x(j),:))
            fprintf('\n')
        end
        fprintf('beta: \n')
        for j=1:size(PART(i).beta,2)
            fprintf('      ')
            fprintf(input.nms(PART(i).beta(j),:))
            fprintf('\n')
        end
        fprintf('eqs:  \n')
        for j=1:size(PART(i).eqs,2)
            fprintf('      ')
            fprintf(input.nms(PART(i).eqs(j),:))
            fprintf('\n')
        end
        fprintf('\n')
		fprintf('-----------------------------------\n')
    end
    fprintf('Parameters not reachable from data available:\n')
    for j=1:size(nc,2)
        fprintf('      ')
        fprintf(input.nms(nc(j),:))
        fprintf('\n')
    end
end

