function L=looph(S,p)
% LOOP_H	L=loop_h(STR,P) Develops a loop hierarchy of the loops on the P
%			cycle partition based on the "inclusion" relationship.  
%			Loop A is considered to be included in loop B if all the vertex
%			of loop A are also elements of loop B.
%		 	STR is expected to have the 7 components generated by the
%			structure.m	function.
%			The result is presented as a level structure matrix L.lev and the 
%			corresponding matrix of inclusions L.str and two custom graphs.

%			(see Warfield pgs. 341-342).

radius=min(nonzeros(S.ecc(p,:)));				% id the radius of the graph
nl=size(nonzeros(S.fbl(:,1,p)),1);				% number of loops -- loops are id by having the first col<>0
incld=sparse(zeros(nl,nl));						% creates a sq. matrix of size=#of loops
							
fprintf('Identifying inclusions within cycle loop set ...\n')

for i=1:nl-1										% for each loop in matrix except the last 
	l=nonzeros(S.fbl(i,:,p));						% id elements of loop
	s(i)=size(l,1);									% capture # of elements (loop lenght)
	e(i)=full(mean(S.ecc(p,l)));					% capture mean eccentricity of loop -- 'full' added to fix v6.5 bug
	for j=i+1:nl									% for all remaining loops
		m=sum(ismember(l,nonzeros(S.fbl(j,:,p))));	% # of elements of l in loop j
		if m==s(i)								% if all elements are members
			incld(j,i)=1;						% include in inclussion matrix
		end;									% end if
	end;										% end for j
end;											% end for i
s(i+1)=size(nonzeros(S.fbl(i+1,:,p)),1);				% caputre length of last loop
e(i+1)=mean(S.ecc(p,nonzeros(S.fbl(i+1,:,p))));

if nnz(incld>0)
	fprintf('Reducing to Adjacency Matrix ...\n')
	str=rtoa(incld);			% determine the adjacency structure of loop set
	fprintf('Calculating Reachability Matrix ...\n')
	R=reach(incld);
	fprintf('Identifying Level Partitions ...\n')
	[lev,cyc]=levels(R);		% determine the level structure for loop set
	
	fprintf('Indexing for plot coordenates ...\n')

	% x coordinates by minimizing the distance to the diagonal
	idx=symrcm(str);
	for i=1:nl
		xy(i,1)=find(idx==i);
	end;
	
	%y coordinates by level
	l=size(lev,1);
	for i=1:l
		temp=find(lev(i,:));
		xy(temp,2)=l+1-i;
	end;

	subplot(2,1,1), gplot(str,xy);
	axis([1 nl 1 l]);
	title('Loop Inclusion Graph (by levels)','FontSize',10,'FontWeight','bold')
	for i=1:nl
		li=int2str(i);
		ll=strcat('[',int2str(s(i)),']');
		msg=strvcat(int2str(i),ll);
		text(xy(i,1)-0.1,xy(i,2)-0.15,msg,'FontSize',9);
	end;
	msg=strvcat('Loop#','[Loop length]');
	text(0,0.5,msg,'FontSize',9);
	axis off;
	
	%x coordintates by loop eccentricity	
	%y coordinates by loop length
	e=e-radius;				% since min(e) is the center of the graph, this readjust
	xy1=[e' s'];
	
	subplot(2,1,2), gplot(str,xy1);
	axis([0 max(e)+1 0 max(s)+1]);
	title('Loop Inclusion Graph','FontSize',10,'FontWeight','bold');
	for i=1:nl
		text(xy1(i,1)-0.1,xy1(i,2)+0.1,int2str(i),'FontSize',9);
	end;
	xlabel('Mean distance form center of cycle','FontSize',10,'FontWeight','bold')
	ylabel('Loop length','FontSize',10,'FontWeight','bold')
	


	
else
	fprintf('No inclusions in loop set ...\n')
	lev=[]; str=[];
end;
L.lev=lev; L.str=str;


function a=rtoa(r)
% R_to_A Deduces the adjacency matrix from a full reachibility matrix.

for i=1:size(r,1)			% for each row in matrix
	for j=find(r(i,:))		% for active elements in row
		if r(i,j)==1		% if element is still active
			r(i,:)=r(i,:)-r(j,:);	% eliminate future dependencies
			for k=find(r(i,:)==-1)	% eliminates -1 in case of mult. paths
				r(i,k)=0;
			end;
		end;
	end;
end;
a=r;


function [levs,cycles] = levels(R)
%LEVELS Level partitions of reachability matrix.
%		[L,C] = levels(R). Only the first element of a cycle is reported
%		in L; the full list of elements in a cycle is reported in C.
%
%		Pgs. 249-263 and 276-284 of Societal Systems

killc=sparse(size(R,1),1); 				% Constant to facilitate removing 
killr=sparse(1,size(R,2));  			% Rows and Columns

k=1; c=1; cycles=[];
control=sparse(ones(1,size(R,2)));		% All elements in the matrix
C=sparse(1,size(R,2));
levs=zeros(1,size(R,2));

while (nnz(control)>0) 					% while there are elements left
	j=1; T=sparse(1,size(R,2));
	for i=find(control)					% for all remaining elements
		P = R(:,i)';					% id predecesors
		if (nnz(P)>0)					% if are any left
			S = R(i,:);					% id succesors
			if (nnz(S&(~(P&S)))==0)		% if no e of S that are ~e of P&S
				T(1,i)=1; 
				if (nnz(P&S)>1)			% We have a cycle!
					fprintf('   Cycle %d ',c)
					fprintf('belongs to level %d.\n',k)
					fprintf('   Identifying Elements of Cycle ...\n')
					cycles(c,:)=S;		% Place it in matrix 
					c=c+1;				% Increment cycle counter
					S(1,i)=0;			% Eliminate original element from loop set
					for m=find(S)		% Remove cycle from R matrix
						R(m,:)=killr;
						R(:,m)=killc;
					end;
					C=C|S;
				end;
			end;
		end;	
	end;
	levs(k,:)=T;						% Once a level is finished put in levs
	k=k+1;								% Increment level counter
	for m=find(T)						% Remove level from R matrix
		R(m,:)=killr;
		R(:,m)=killc;
	end;
	control=control&(~(T|C));
end;
levs=sparse(levs);
cycles=sparse(cycles);
