function S=structure(input,MILS,outpt)
% STRUCTURE S=structure(ADJ[,MILS,output])
%       Decomposes the structure of a system described as an
%       adjacency binary matrix based on R. Oliva, Model structure
%       analysis through grraph theory: partition heuristics and 
%       feedback structure decomposition, 2003.
%       Returns an arrayed structure with five components: 
%       S.adj, S.lev, S.cyc, S.fbl, S.exc and S.idx.
%       if ADJ is a structured array with the vector with variable
%       (.nms), the function also returns S.nms.
%       If output==0 progress reports are suppressed. 
%
%	.adj  replica of the input variable.
%	.nms  replica of the input variable.
%	.lev  and CYC are sparse binary matrices. Each row identifies
%	      the elements that belong to a partition or cycle.
%	.cyc  for each cycle partition: is a matrix that in each row 
%	      has the ordered set of elements in a feedback loop. 
%	.fbl  is the shortest independen loop set ordered by loop length.
%	      if MILS==1, the ILS is the minimal shortest ILS.
%	.exc  for each cycle partition: is a row where cell(i) contains
%	      the excentricity of the ith node -- 0 if not in partition. 
%	.idx  is an index vector ordering the elements by level and 
%	      minimizing the distance of iteractions to the main diagonal.


% The main function is structure and it does the identification in
% four stages.
%		a) Identifies the level partition of the system
%			(Pgs 249-263 and 276-284) 
%		b) Identifies maximal cycle sets out of level partitions
%			and reports membership of cycle to level.
%			(Pgs 253-256)
%		c) Identifies feed back loops out of maximal cycle sets 
%			(Pgs 336-341) and reports node excentricity within cycle.
%		d) Identifies Independent loop set, orders it, and reports
%			reduction ratio.
%
%  											Rogelio Oliva
%											September 29, 2003.	
%											Boston, MA
%================================================================

if isa(input,'struct') 
	a=input.adj; 
else
	a=input;
end;

if nargin>=2 && MILS~=0     % activate only iff explicitly required
    MILS=1;
else
    MILS=0;
end;

global output;				% define as global variable	
if nargin==3 && outpt==0   % activate only iff explicitly required
    output=0;
else
    output=1;
end;

if output fprintf('Calculating Reachability Matrix ...\n'), end
R=reach(a);
if output fprintf('Identifying Level Partitions ...\n'), end
[lev,cyc]=levels(R);
if (size(cyc,1)>0) lev=integ(lev,cyc); end; % if cycles in model, integrate cyc elements in lev

for i=1:size(cyc,1)							% for each maximal cycle identified
	m=1; y=[];								% initialize loop counter and looplist
	index=find(cyc(i,:))';					% create index-row w/cycle elements
	if output fprintf('  Identifying Loops within Cycle %d ...\n',i), end
	c_elm=find(cyc(i,:));
	[E,fc]=loops(a(c_elm,c_elm));			% id the fb loops in cycle and reportns node excentricity
	ecc(i,c_elm)=E';						% place excentricities in a vector corresponding to the original node number
	
	% The following for cycle translates the fc back to the original node number
	% This can be probably improved using a similar technique than for E above
	for j=1:size(fc,1)						% for each fb loop identified
		for k=1:size(fc,2)					% for each element id in the loop
			if (fc(j,k)~=0)					% if there is an element active
				y(m,k)=index(fc(j,k));	    % extract its coodrinates form index
			else							% else, end of fbl ... move to next
				break;
			end;
		end;
		m=m+1;
	end;
	
	if ~isempty(y)
        if output fprintf('    Reducing Cycle %d to Shortest Independent Loop Set ...\n',i), end
		y=ils(y,a,1); 
		if MILS==1
			if output fprintf('    Reducing Cycle %d to Minimal Shortest Independent Loop Set ...\n',i), end
			y=ils(y,a,-1);
        end;
		fbl(1:size(y,1),1:size(y,2),i)=y(1:size(y,1),1:size(y,2));
%		Edges=nnz(bsub(a(c_elm,c_elm),eye(size(c_elm,2),size(c_elm,2))));
%		Vertex=size(c_elm,2);
%		fprintf('    E V R r:    %d %d %d %d \n', Edges, Vertex, Edges-Vertex+1, size(y,1))
    end;
end;

if output fprintf('Indexing Variables by Level ...\n'), end
idx=lindex(lev,cyc,a);
ecc=sparse(ecc);
S.adj=a;
if isa(input,'struct') S.nms=input.nms; end;
clear global output;
S.lev=lev; S.cyc=cyc; S.ecc=ecc; S.fbl=fbl; S.idx=idx;


function [levs,cycles] = levels(R)
%LEVELS Level partitions of reachability matrix.
%		[L,C] = levels(R). Only the first element of a cycle is reported
%		in L; the full list of elements in a cycle is reported in C.
%
%		Pgs. 249-263 and 276-284 of Societal Systems

global output;							% refer to global output control variable
killc=sparse(size(R,1),1); 				% Constant to facilitate removing 
killr=sparse(1,size(R,2));  			% Rows and Columns

k=1; c=1; cycles=[];
control=sparse(ones(1,size(R,2)));		% All elements in the matrix
C=sparse(1,size(R,2));
levs=zeros(1,size(R,2));

while (nnz(control)>0) 					% while there are elements left
	j=1; T=sparse(1,size(R,2));
	for i=find(control)					% for all remaining elements
		P = R(:,i)';					% id predecesors
		if (nnz(P)>0)					% if are any left
			S = R(i,:);					% id succesors
			if (nnz(S&(~(P&S)))==0)		% if no e of S that are ~e of P&S
				T(1,i)=1; 
				if (nnz(P&S)>1)			% We have a cycle!
					if output 			%
     					fprintf('  Cycle %d ',c)
						fprintf('belongs to level %d.\n',k)
					end;
                    cycles(c,:)=S;		% Place it in matrix 
					c=c+1;				% Increment cycle counter
					S(1,i)=0;			% Eliminate original element from loop set
					for m=find(S)		% Remove cycle from R matrix
						R(m,:)=killr;
						R(:,m)=killc;
					end;
					C=C|S;
				end;
			end;
		end;	
	end;
	levs(k,:)=T;						% Once a level is finished put in levs
	k=k+1;								% Increment level counter
	for m=find(T)						% Remove level from R matrix
		R(m,:)=killr;
		R(:,m)=killc;
	end;
	control=control&(~(T|C));
end;
levs=sparse(levs);
cycles=sparse(cycles);


function  y = integ(levels,cycles)
% INTEG(lev,cyc)	Integrates the elements of a cycle
%					partition into its corresponding level partition
%					Returns the adjusted lev structure

j=1;
for i=1:size(cycles,1)				% for every cycle partition
	c=find(cycles(i,:));			% id its elements
	while (j<= size(levels,1))		% while there are levels to compare
		l=find(levels(j,:));		% id elements of level
		if ~isempty(intersect(l,c)) % if there is at least one element in common
			for k=c					% for every element in cycle
				levels(j,k)=1;		% include in level
			end;
			break;					% break from while
		end;	
		j=j+1;						% next level
	end;
end;
y=levels;



function [E,y]=loops(a)
% LOOPS(a)	Identifies the gedetic cycles within a 
%			maximal cycle set, ie, a universal reachability matrix
%
%			Pgs. 336-341 Societal Systems

D=vrtx_dist(a);
E=max(D,[],2);								% Excentricity is the longest (max) of the shortest distance between nodes
GP=tril(D)'+triu(D);						% Lenght of cycle by adding paths a->b and b->a
maxloop=max(max(GP));

k=1;
for i=2:maxloop								% for all potential loop sizes
	[r,c]=find(GP==i);						% id elements involved in such loops
	for j=1:size(r,1)   					% for all elements of loops same size
		if (r(j)~=0)    					% Check if loop has not been cleared
			loop=track_vrtx(r(j),c(j),D);
			for m=find(ismember(r(j+1:size(r,1)),loop))'	% for the roots of loops of same length in loop
				if size(m,1)>0								% extra if added for v 6.5 compatibility  -- in old ver. empty m would not enter for
					if ~isempty(find(loop==c(j+m)))				% if pair is allready id
						r(j+m)=0;								% remove from list
					end;
				end;
			end;
			if size(loop,2) == size(unique(loop),2) 		% enter only if not figure 8
				y(k,1:size(loop,2))=loop;	
				k=k+1;
			end;			
		end;
	end;
end;


function y = vrtx_dist(a)
% DIST(m).		Calculates the reachability distance matrix
%				from a binary threshold matrix.
%
%				Pg. 336 Societal Systems

A=bsum(a,eye(size(a)));  	%Identity added to initial matrix
G=bsub(A,eye(size(a)));	 	%ensure that main diagonal is clear

Bprev=A;
for i=2:size(a,1)			%Formula Pg. 336
	Bcur=bprd(A,Bprev);
	G=G+i*(Bcur-Bprev);
	Bprev=Bcur;
end;
y=G;


function y=track_vrtx(u,v,dmatrix)
% The track-vrtx function derives "one" of the loops from vertex
% u through vertex v in a distance matrix.
% Pg. 340 Societal Systems

k=1;	x(k)=u;									% The u->v path
for j=1:(dmatrix(u,v)-1) 						% for 1 to (lenght of u->v path)-1
	rh=find(dmatrix(:,v) == (dmatrix(u,v)-j))'; % pred of v with step=length(u->v)-j
	lh=find(dmatrix(x(k),:) == 1); 				% succ of last element in path step=1			
	k=k+1;		
	for i=1:size(rh,2)
		if (find(lh == rh(1,i)))
			x(k)=rh(1,i);
			break;
		end;
	end;
end;	
								
k=k+1;	x(k)=v;									% The v->u path
for j=1:(dmatrix(v,u)-1)	 					% for 1 to (lenght of v->u path)-1
	rh=find(dmatrix(:,u) == (dmatrix(v,u)-j))';	% pred of u with step=length(v->u)-j
	lh=find(dmatrix(x(k),:) == 1); 				% succ of last element in path step=1			
	k=k+1;
	for i=1:size(lh,2)
		if (find(rh == lh(1,i)))
			x(k)=lh(1,i);
			break;
		end;
	end;
end;
y=x;


function y=ils(f,a,order)
% ILS	Independent Loop Set.
%		ILS(f,a) reduces a set of feedback loops in a system to the maximal
%		number of independent loops. The f matrix has each loop in a row
%		-- assending in lenght -- with the elements listed across in columns.
%		The a matrix is the full adjaciancy matrix.

b=(sparse(zeros(size(a))));				% matrix w/zeros to control base
k=1;									% counter for output matrix
S=ones(1,size(f,1));                    % intialize loop set to contain all loops

a=bsub(a,speye(size(a)));				% eliminate self refering equations for loop search

E(size(f,1))=struct('edges',sparse(zeros(size(a))));                 % initialize space for loop edges
for i=1:size(f,1)						% for each loop i in f
	length=nnz(f(i,:));						% identify its length
	T=sparse(zeros(size(a)));
	for j=1:length-1						% for each vertex in loop i
		T(f(i,j),f(i,j+1))=1;					% id edge between edge and next
	end;
	T(f(i,length),f(i,1))=1;				% id edge from last to first node
	E(i).edges=T;
end;
                                
while nnz(S)>0							% while loops in length set
	for j=find(S)						    % for every loop left
		c(j)=nnz((E(j).edges-b)>0); 			% id # of new edges contributed by loop
        if c(j)==0	S(j)=0;	 end;			    % if loop does not contribute remove it from set
    end;
	if order==-1 m=max(nonzeros(c));	    % if inverse order id maximal contribution           
	else m=min(nonzeros(c)); end;			% else id minimal contribution
	if m>0							        % if at least one contribution 
		m=find(c==m);                           % id loops with extreme contribution
        if order==-1  m=fliplr(m); end;         % flip so longest loop is first
		m=m(1);                                 % truncate pointer to the fist loop (shortest or longest)
		b=bsum(b,E(m).edges);						% add edges to base
		l(k,:)=f(m,:);							% add loop to output list
		k=k+1;									% increment loop counter
		S(m)=0;									% remove loop from length set
		c(m)=0;                                 % clear contribution vector
	end;	
end;	

[r,c]=find(l);		
l=l(1:max(r),1:max(c));					% clip the matrix to minimum size 
										% sort output by length
for i=1:size(l,1) 						% for all loops on list
	l_length(i)=nnz(l(i,:)); 				% id their length
end;
k=1;									% initialize output counter
for i=sort(l_length);					% for ordered length
	j=find(l_length==i);					% find loop
	j=j(1);									% truncate
	y(k,:)=l(j,:);							% exchange
	k=k+1;
	l_length(j)=0;
end;


function gct=lindex(l,c,s)
% LINDEX gct=lindex(lev,cyc,adj)
%		 Generates a strcuture with the information for to control the
%		 graphic display of the structure.  The first element (idx) contains a
%	 	 vector index ordered by level --including the  cycle elements within level.
%		 Within each level it orders elements to minimize bandwith
%		 --the distance from an interaction to the main diagonal.
%		 The secont element (tks) contains the tickmarks for the level partitions.
%		 The third element (lbl) contain the labels for the tickmarks.

ticks=[]; count=0;					% initialize tick vector and counter
li=[];	labels=[];					% initialize index and labels vectors
k=1;								% initialize cycle counter
nc=find(c(k,:));					% identify elements of first cycle

cycleflag=0;						% set cycle flag in off position
for i=1:size(l,1)					% for each level
	le=find(l(i,:));				% extract level elements
	if ~isempty(intersect(le,nc))	% if current cycle and level share at least one element
		le=union(le,nc);				% elements of cycle are included in level
		cycleflag=1;					% mark that cycle has been found
		k=k+1;							% increase cycle counter
		if k<=size(c,1)					% if cycles are left
			nc=find(c(k,:));			% identify elements of next cycle
		else			
			nc=[];						% clear elements of cycle
		end;
	end;
	count=count+size(le,2);			% add # of elements to tick counter
	ticks=[ticks count];			% append counter to tick vector
	label=sprintf('L%2.0f',i);		% identify level
	if cycleflag					% if loop was appended
		label=sprintf('C%2.0f',k-1);	% label identifies loop	
		cycleflag=0;					% clear flag
	end;
	labels=[labels;label];
	o=symrcm(s(le,le));				% sort level elements to minimize distance in s
	li=[li,le(o)];					% append sorted level to the index vector
end;
ticks=ticks+0.5;					% shift ticks half a point
ticks=ticks(1,1:size(ticks,2)-1);		% drop last tick
gct.idx=li; gct.tks=ticks; gct.lbl=labels;


function y=bsum(a,b)
% BSUM Y=BSUM(A,B)
%	   Performs the binary addition of two matrices.
%	   Matrices have to be the same size.
y=(a+b);
c=find(y>1);			%id nonzero elements that are not one	
y(c)=ones(size(c));		%truncate back to ones


function y=bsub(a,b)
% BSUB Y=BSUB(A,B)	
%	   Performs the binary subtraction (A-B) of two matrices.
%	   Matrices have to be the same size and the first matrix
%	   has to be larger, on every element, than the second.
y=(a-b);
c=find(y<0);			%id negative values
y(c)=zeros(size(c));	%truncate back to zeros


function y=bprd(a,b)
% BPRD Y=PBRD(A,B)
%	   Performs the binary product of two matrices.
%	   The number of rows on the first factor has to match the 
%	   number of columns on the second.
y =(a*b);
c=find(y>1);			%id nonzero elements that are not one	
y(c)=ones(size(c));		%truncate back to ones
